// TermHire.cpp
// v0.98
// version for AADR9 release of the BeOS
// release date 30/05/97

#include <Directory.h>
#include <Window.h>
#include <StringView.h>
#include <File.h>
#include <Alert.h>
#include <Message.h>
#include <Errors.h>
//#include <Debug.h>
#include <Path.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#pragma once
// use this define if you want little windows to explain you what's going on
//#define VERBOSE 1


// global variables. I know it is ugly, but this is just an add-on

char *argv[4] = { "/boot/apps/Terminal" ,"/boot/bin/sh","-login",NULL}; 
// 1st arg : the app which is launched
// 2nd arg : tells it which shell to use. It uses the SHELL variable,
//           but if it is not set, the default "/boot/bin/sh" value is used.
// 3rd arg : tells the shell to be a login shell.

#define ARGC 3 // the number of arguments in argv
int numOK = 0; // number of Terminals launched correctly
extern char **environ; // this is the global context of the shell

#ifdef VERBOSE
BWindow *win;
BStringView *status;
#endif


#pragma export on
void process_refs(entry_ref dir_ref, BMessage* msg, void*);
#pragma export reset

// function prototypes
int LaunchTerminal(const char *path);
char **new_environ(char **orig, char *new_var);
char *give_value(char *variable_name, char **environ);

static void error(const char *message)
{
	BAlert *alert = new BAlert("TermHire Error", message, "OK");
	alert->Go();
}

// this function creates the new environment
// which is a copy of the "orig" environment, 
// plus the environment variable defined in "new_var"
char **new_environ(char **orig, char *new_var)
{
	int nb_lines=0,i;
	char **dest;
	
	while(orig[nb_lines]!=NULL) nb_lines++;
	dest=(char **)malloc((nb_lines+2)*sizeof(char **));
	
	// we just need to copy the pointers, since there will be another copy
	// when the app will be launched(I'm not really sure of that though)
	for(i=0;i<nb_lines;i++)
		dest[i]=orig[i];
		
	// now the new one is allocated	
	dest[nb_lines]=(char *)malloc(strlen(new_var)+1);
	strcpy(dest[nb_lines],new_var);
	dest[nb_lines+1]=NULL;
	
	return dest;
}

// this function return the string value of the given environment
// "environ" related to the name "variable_environment"
// a NULL pointer is returned if the variable is not found
char *give_value(char *variable_name, char **environ)
{
	int i=0; // string number
	char *search_string;
	char *resultat;
	bool found_it=false;
	
	search_string=(char *)malloc((strlen(variable_name)+1)*sizeof(char));
	strcpy(search_string, variable_name);
	strcat(search_string, "=");
	while( !found_it && (environ[i]!=NULL) )
	{
		if(strncmp(search_string, environ[i], strlen(search_string))==0)
			found_it = true;
		else
			i++;
	}
	if(found_it)
	{
		resultat = (char *)malloc((strlen(environ[i])-strlen("SHELL="))*sizeof(char));
		strcpy(resultat, &environ[i][strlen("SHELL=")*sizeof(char)]);
		return resultat; // why don't you use the standard shell anyway, hmmm ?
	}
	else
		return NULL; // :-/ take default
}

// this function launch a Terminal in the specified path
int LaunchTerminal(const char *path)
{
	char **environ2,path2[MAXPATHLEN + 13];
	char errorstring[MAXPATHLEN + 40];

	// now, it formats the new environment context,
	// containing the variable FOLDER_PATH
	path2[0]='\0';
	strcat(path2,"FOLDER_PATH=");
	strcat(path2,path);
	// the "environ" variable is a string array that ends with a NULL
	// pointer and contains all the variables defined at boot time
	// in the environment
	environ2=new_environ(environ,path2);

	// the info window stuff
#ifdef VERBOSE
	win->Lock();
	status->SetText(path);
	win->Unlock();
#endif

	thread_id tid;
	// we launch the new Terminal in a separate thread
	tid = load_image(ARGC, argv, environ2);
	if (tid > 0)
	{
		// the launch was successful, now run the thread
		resume_thread(tid);
		numOK++;
		return 1;
	}
	else 
	{
		// the launch was unsuccessful
		sprintf(errorstring, "Error %08x running %s", tid, argv[0]);
		error(errorstring);
		return B_ERROR;
	}
}

// this is the function the Tracker call when the add-on is selected
// dir_ref is the folder you have launched it from
// msg is a BMessage containing the references to BEntry's(Files/Folders)
// and for "void *", I don't know yet...
void process_refs(entry_ref dir_ref, BMessage* msg, void*)
{
	long count; // number of references stored in the BMessage
	ulong type; // type of the BMessage
	BPath path; // path to use to launch the terminal
	char *shellpath; // temp string containing the path of the shell

// this creates an informative window about the completion of the add-on
#ifdef VERBOSE
	char strpath[1000];
	BRect area(0,0, 250,15);
	screen_info info;

	get_screen_info(&info);
	area.OffsetBy(info.frame.Width()/2-125, info.frame.Height()/2-10);
	win = new BWindow(area, "TermHire Progress", B_TITLED_WINDOW,
		B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_NOT_CLOSABLE);
	status = new BStringView(win->Bounds(), "status", "TermHire");
	win->AddChild(status);
	win->Show();
#endif

	// we set up the shell path/name by grabbing it in the environment
	shellpath=give_value("SHELL",environ);
	if(shellpath!=NULL) argv[1]=shellpath;

	// first, we consider the case when the add-on has been selected
	// from the file menu of the browser window or by right-clicking
	// in the background of the window
	// i.e : in this case, there is no refs on the BMessage
	msg->GetInfo("refs", &type, &count);
	if (count == 0)
	{
		BEntry entry(&dir_ref);
		entry.GetPath(&path);
		if(!LaunchTerminal(path.Path())) goto exit;
	}
	else
	{
		BDirectory dir;
		BEntry entry;
		// if there are datas on the BMessage, but not from the
		// directory type, exit...
		if (dir.SetTo(&dir_ref) != B_NO_ERROR) 
		{
			error("You must launch TermHire from folders.");
			return;
		}
		if(dir.GetEntry(&entry) != B_NO_ERROR)
		{
			error("Sorry, I cannot access the folder while launching"
			      "a Terminal");
			return;
		}
		entry_ref ref;
		BFile file;
		for (int ix=0; ix<count; ix++)
		{
			if(msg->FindRef("refs", ix, &ref) == B_NO_ERROR)
			// if the ref is a folder, launch a Terminal on this folder
			if(dir.SetTo(&ref) == B_NO_ERROR)
			{
				entry.SetTo(&ref);
				entry.GetPath(&path);
				if(!LaunchTerminal(path.Path())) goto exit;
			}
			// if the ref is not a folder, launch a Terminal on the parent folder
			else if(entry.SetTo(&ref) == B_NO_ERROR) 
			{
				// I ask for the parent directory of the entry
				entry.GetParent(&entry);
				entry.GetPath(&path);
				if(!LaunchTerminal(path.Path())) goto exit;
			}
		}
	}
	
#ifdef VERBOSE
	sprintf(strpath, "%d out of %d were set", numOK, count);
	win->Lock();
	status->SetText(strpath);
	win->Unlock();
#endif
exit:
#ifdef VERBOSE
	snooze(100000);
	win->Quit();
#endif
	;
}